@@ -96,6 +96,7 @@ __SWITCH_OPTIONS__ = {
     "OS.UpdateRdmaDriver": False,
     "OS.CheckRdmaDriver": False,
     "Logs.Verbose": False,
+    "Logs.Console": True,
     "Extensions.Enabled": True,
     "Provisioning.Enabled": True,
     "Provisioning.UseCloudInit": False,
@@ -108,6 +109,7 @@ __SWITCH_OPTIONS__ = {
     "DetectScvmmEnv": False,
     "ResourceDisk.Format": False,
     "ResourceDisk.EnableSwap": False,
+    "ResourceDisk.EnableSwapEncryption": False,
     "AutoUpdate.Enabled": True,
     "EnableOverProvisioning": True,
     "CGroups.EnforceLimits": False,
@@ -172,10 +174,14 @@ def enable_rdma(conf=__conf__):
 def enable_rdma_update(conf=__conf__):
     return conf.get_switch("OS.UpdateRdmaDriver", False)
 
+def enable_check_rdma_driver(conf=__conf__):
+    return conf.get_switch("OS.CheckRdmaDriver", True)
 
 def get_logs_verbose(conf=__conf__):
     return conf.get_switch("Logs.Verbose", False)
 
+def get_logs_console(conf=__conf__):
+    return conf.get_switch("Logs.Console", True)
 
 def get_lib_dir(conf=__conf__):
     return conf.get("Lib.Dir", "/var/lib/waagent")
@@ -324,7 +330,9 @@ def get_resourcedisk_format(conf=__conf__):
 
 def get_resourcedisk_enable_swap(conf=__conf__):
     return conf.get_switch("ResourceDisk.EnableSwap", False)
-
+    
+def get_resourcedisk_enable_swap_encryption(conf=__conf__):
+    return conf.get_switch("ResourceDisk.EnableSwapEncryption", False)
 
 def get_resourcedisk_mountpoint(conf=__conf__):
     return conf.get("ResourceDisk.MountPoint", "/mnt/resource")
